;; Fundamental classes
(IN-PACKAGE "COMMON-LISP-USER")


(DEFUN Zp-FUNDAMENTAL-GMSM (p dmns pi-elm)  ;; pi-elm not "equal" to 0
   (declare (fixnum dmns pi-elm))
   (the gmsm
      (if (= 1 dmns)
         (list pi-elm)
         (let ((bspn (if (= 2 dmns) nil +null-gbar+)))
            (declare (type gmsm bspn))
            (make-gbar :dmns dmns
               :list (cons (absm 0 (Zp-fundamental-gmsm p (1- dmns) pi-elm))
                        (mapcar
                           #'(lambda (k)
                                (declare (fixnum k))
                                (absm (mask k) bspn))
                           (nreverse (<a-b> 0 (- dmns 2))))))))))
#|
  (Z2-fundamental-gmsm 1 1)
  (Z2-fundamental-gmsm 2 1)
  (Z2-fundamental-gmsm 3 1)
  (Z2-fundamental-gmsm 4 1)
  (Zp-fundamental-gmsm 2 1 1)
  (Zp-fundamental-gmsm 2 2 1)
  (Zp-fundamental-gmsm 2 3 1)
  (Zp-fundamental-gmsm 2 4 1) 

  (Zp-fundamental-gmsm 5 1 1)
  (Zp-fundamental-gmsm 5 4 1)
  
|#


(DEFUN Zp-COCYCLE-GBAR (p n dmns cocycle)
  ;; cocycle \in Z^n(\Delta^{dmns}, \pi) with \pi = Z/pZ
  (declare
    (fixnum p n dmns)
    (list cocycle))
  (the absm
    (progn
      (mapc
        #'(lambda (cons)
	    (declare (cons cons))
            (setf (cdr cons)
                  (mod (cdr cons) p)))
       cocycle)
      (cond ((= n 1)
	     (g-bar-absm (cyclicgroup p) (nreverse (mapcar #'cdr cocycle))))
	    ((= n dmns)
	     (let ((value (cdr (first cocycle))))
	       (declare (fixnum value))
	       (if (zerop value)
		   (absm (mask dmns) +null-gbar+)
		 (absm 0 (zp-fundamental-gmsm p dmns value)))))
	    (t
	     (let ((cocycle1 +empty-list+)
		   (cocycle2 +empty-list+))
	       (declare (list cocycle1 cocycle2))
	       (dolist (cons cocycle)
		 (declare (cons cons))
		 (if (evenp (car cons))
		     (push (cons (ash (car cons) -1) (cdr cons))
			   cocycle1)
		   (push (cons (ash (car cons) -1) (cdr cons))
			 cocycle2)))
	       (setf cocycle1 (nreverse cocycle1)
		     cocycle2 (nreverse cocycle2))
	       (mapc
		#'(lambda (cons)
		    (declare (cons cons))
		    (when (evenp (car cons))
			  (setf (cdr cons)
				(mod (- (cdr cons)
					(cdr (assoc (1+ (car cons)) cocycle1)))
				     p))))
		cocycle2)
	       (let ((head-absm (zp-cocycle-gbar p (1- n) (1- dmns) cocycle2))
		     (tail-absm (zp-cocycle-gbar p n (1- dmns) cocycle1)))
		 (declare (type absm head-absm tail-absm))
		 (normalize-gbar
		  (cons dmns
			(cons head-absm
			      (rest (unnormalize-gbar tail-absm
						      (if (= n 2) () +null-gbar+)))))))))))))

#|
  (cat-init)
  (setf d (delta 10))
  (setf chml-clss
    (build-mrph :sorc d :trgt (z-chcm) :degr -1
                :intr #'(lambda (dmns gmsm)
                          (term-cmbn 0 (mod gmsm 2) :gnrt-z))
                :strt :gnrt
                :orgn '(essai-1)))
  (gmsm-cocycle (face d) 1 4 31 chml-clss)
  (zp-cocycle-gbar 5 1 4 *)
  (zp-cocycle-gbar 2 2 4 **) 
  (setf chml-clss
    (build-mrph :sorc d :trgt (z-chcm) :degr -2
                :intr #'(lambda (dmns gmsm)
                          (term-cmbn 0 (mod gmsm 2) :gnrt-z))
                :strt :gnrt
                :orgn '(essai-2)))
  (gmsm-cocycle (face d) 2 2 7 chml-clss)
  (z2-cocycle-gbar 2 2 *)
  (gmsm-cocycle (face d) 2 2 0 chml-clss) ;; normally illegal
  (z2-cocycle-gbar 2 2 *)
  (gmsm-cocycle (face d) 2 3 15 chml-clss)
  (z2-cocycle-gbar 2 3 *)
  (gmsm-cocycle (face d) 2 4 31 chml-clss)
  (z2-cocycle-gbar 2 4 *)
  (setf chml-clss
    (build-mrph :sorc d :trgt (z-chcm) :degr -3
                :intr #'(lambda (dmns gmsm)
                          (term-cmbn 0 (mod gmsm 2) :gnrt-z))
                :strt :gnrt
                :orgn '(essai-3)))
  (gmsm-cocycle (face d) 3 4 31 chml-clss)
  (z2-cocycle-gbar 3 4 *)
|#

(DEFUN Zp-COCYCLE-GBAR-HEAD (p n dmns cocycle)
  ;; cocycle \in Z^n(\Delta^{dmns}, \pi) with \pi = Z/pZ
  (declare
    (fixnum n dmns)
    (list cocycle))
  (the absm
    (progn
      (mapc
        #'(lambda (cons)
	    (declare (cons cons))
            (setf (cdr cons)
                  (mod (cdr cons) p)))
       cocycle)
      (cond ((= n 1)
	     (error "In Zp-COCYCLE-GBAR-HEAD, this point should not have been reached."))
		 ; (zp-bar-absm (nreverse (mapcar #'cdr cocycle))))
	    ((= n dmns)
	     (let ((value (cdr (first cocycle))))
	       (declare (fixnum value))
	       (if (zerop value)
		   (if (= n 2)
		       (absm 1 +empty-list+)
		     (absm (mask (1- dmns)) +null-gbar+))
		 (absm 0 (zp-fundamental-gmsm p (1- dmns) value)))))
	    (t
	     (let ((cocycle1 +empty-list+)
		   (cocycle2 +empty-list+))
	       (declare (list cocycle1 cocycle2))
	       (dolist (cons cocycle)
		 (declare (cons cons))
		 (if (evenp (car cons))
		     (push (cons (ash (car cons) -1) (cdr cons))
			   cocycle1)
		   (push (cons (ash (car cons) -1) (cdr cons))
			 cocycle2)))
	       (setf cocycle1 (nreverse cocycle1)
		     cocycle2 (nreverse cocycle2))
	       (mapc
		#'(lambda (cons)
		    (declare (cons cons))
		    (when (evenp (car cons))
			  (setf (cdr cons)
				(mod (- (cdr cons)
					(cdr (assoc (1+ (car cons)) cocycle1)))
				     p))))
		cocycle2)
	       (zp-cocycle-gbar p (1- n) (1- dmns) cocycle2)))))))

#|
  (cat-init)
  (setf d (delta 10))
  (setf chml-clss
    (build-mrph :sorc d :trgt (z-chcm) :degr -1
                :intr #'(lambda (dmns gmsm)
                          (term-cmbn 0 (mod gmsm 2) :gnrt-z))
                :strt :gnrt
                :orgn '(essai-1)))
  (gmsm-cocycle (face d) 1 4 31 chml-clss)
  (z2-cocycle-gbar 1 4 *)
  (z2-cocycle-gbar-head 1 4 **)
  (setf chml-clss
    (build-mrph :sorc d :trgt (z-chcm) :degr -2
                :intr #'(lambda (dmns gmsm)
                          (term-cmbn 0 (mod gmsm 2) :gnrt-z))
                :strt :gnrt
                :orgn '(essai-2)))
  (gmsm-cocycle (face d) 2 2 7 chml-clss)
  (z2-cocycle-gbar 2 2 *)
  (z2-cocycle-gbar-head 2 2 **)
  (gmsm-cocycle (face d) 2 2 0 chml-clss) ;; normally illegal
  (z2-cocycle-gbar 2 2 *)
  (z2-cocycle-gbar-head 2 2 **)
  (gmsm-cocycle (face d) 2 3 15 chml-clss)
  (z2-cocycle-gbar 2 3 *)
  (z2-cocycle-gbar-head 2 3 **)
  (gmsm-cocycle (face d) 2 4 31 chml-clss)
  (z2-cocycle-gbar 2 4 *)
  (z2-cocycle-gbar-head 2 4 **)
  (setf chml-clss
    (build-mrph :sorc d :trgt (z-chcm) :degr -3
                :intr #'(lambda (dmns gmsm)
                          (term-cmbn 0 (mod gmsm 2) :gnrt-z))
                :strt :gnrt
                :orgn '(essai-3)))
  (gmsm-cocycle (face d) 3 4 31 chml-clss)
  (z2-cocycle-gbar 3 4 *)
  (z2-cocycle-gbar-head 3 4 **)
|#




(DEFUN Zp-WHITEHEAD-SINTR (p smst n chml-clss
				&aux (face (face smst))
				     (k-zp-n-1 (k-zp p (1- n)))
				     (idnt (bspn k-zp-n-1))
				     (efhm (efhm smst)))
   (declare
      (type simplicial-set smst)
      (fixnum n)
      (type morphism chml-clss)
      (type face face)
      (type ab-simplicial-group k-zp-n-1)
      (type gmsm idnt)
      (type equivalence efhm))
   ;;(setf chml-clss
	;; (i-cmps chml-clss (rf efhm) (lg efhm)))
   (flet ((rslt (dmns gmsm)
	    (declare
	      (fixnum dmns)
	      (type gmsm gmsm))
	    (if (< dmns n)
	        (absm (mask (1- dmns)) idnt)
	      (zp-cocycle-gbar-head p n dmns
		(gmsm-cocycle face n dmns gmsm chml-clss)))))
     (the sintr #'rslt)))

(DEFUN Zp-WHITEHEAD (p smst chml-clss &aux (n (- (degr chml-clss))))
   (declare
      (type simplicial-set smst)
      (fixnum p n)
      (type morphism chml-clss))
   (the fibration
      (build-smmr
         :sorc smst :trgt (k-zp p (1- n)) :degr -1
         :sintr (Zp-whitehead-sintr p smst n chml-clss)
         :orgn `(Zp-whitehead ,p ,smst))))



#|
(cat-init)
(setf k-z3-1 (k-zp 3 1))
(setf chml-clss (chml-clss k-z3-1 3))
;;(setf chml-clss
;;	 (i-cmps chml-clss (rf (efhm k-z3-1)) (lg (efhm k-z3-1))))
(setf tau (zp-whitehead 3 k-z3-1 chml-clss))
(setf x (fibration-total tau))
(homology x 0 5)
(homology x 5)


(cat-init)
(setf k-z3-1 (k-zp 3 1))
(setf chml-clss (chml-clss (k-zp-1 3) 3))
(n-mrph 3 *)
(setf tau (zp-whitehead 3 k-z3-1 *))
(setf x (fibration-total tau))
(homology x 0 5)
;;(homology x 5)
(dotimes (i 5) (print (length (basis x i))))

(cat-init)
(setf k-z3-3 (k-zp 3 3))
(setf chml-clss (chml-clss k-z3-3 3))
(setf tau (zp-whitehead 3 k-z3-3 *))
(setf x (fibration-total tau))
(homology x 0 5)




(homology k-z3-1 0 5)


(cat-init)
(setf k-z3-1 (k-zp 3 1))
(setf k-z3-2 (k-zp 3 2))
(setf x (crts-prdc k-z3-1 k-z3-2))
(homology x 0 5)


(cat-init)
(setf x1 (k-z 2))
(chml-clss x1 2)
(n-mrph 3 *)
(z-whitehead x1 *)
(fibration-total *)
(setf x2 *)
(homology x2 0 8)
(chml-clss x2 4)
(z-whitehead x2 *)
(setf x3 (fibration-total *))
(homology x3 0 5)


(setf y2 (k-zp 3 1))
(homology y2 0 8)
(chml-clss y2 4)
(z-whitehead y2 *)
(setf y3 (fibration-total *))
(homology y3 0 5)

(setf ey2 (echcm y2))
(dotimes (i 6) (print (length (basis ey2 i))))
(dotimes (i 6) (print (chcm-mat ey2 i)))
(setf ch2 (chml-clss y2 4

(chml-clss x2 3)
(z-whitehead x2 *)
(setf x3 (fibration-total *))
(homology x3 0 5)

 

(cat-init)
(setf kz21 (k-z2 1))
(chml-clss kz21 3)
(z2-whitehead kz21 *)
(setf X (fibration-total *))
(homology X 0 5)


(cat-init)
(setf kz21 (k-zp 2 1))
(chml-clss kz21 3)
(zp-whitehead 2 kz21 *)
(setf X (fibration-total *))
(homology X 0 5)




(cat-init)
(setf x1 (k-z 2))
(chml-clss x1 2)
(n-mrph 2 *)
(z-whitehead x1 *)
(fibration-total *)
(setf x2 *)
(homology x2 0 6)

(chml-clss x2 4)
(z-whitehead x2 *)
(setf x3 (fibration-total *))
(homology x3 0 5)

(chml-clss x3 3)
(n-mrph 2 *)
(z-whitehead x3 *)
(setf x4 (fibration-total *))
(homology x4 0 5)






|#